using System;

namespace RoboidControl {

    /// <summary>
    /// A sensor measuring distance
    /// </summary>
    public class DistanceSensor : Thing {

        /// <summary>
        /// Create a new distance sensor
        /// </summary>
        /// <param name="parent">The parent thing</param>
        public DistanceSensor(Thing parent) : base(parent) {
            this.type = Type.DistanceSensor;
            this.name = "Distance sensor";
        }

        public float _internalDistance = float.PositiveInfinity;
        public float internalDistance {
            get { return _internalDistance; }
            set {
                if (value != _internalDistance) {
                    _internalDistance = value;
                    distanceUpdated = true;
                    owner.Send(new BinaryMsg(this));
                }
            }
        }
        protected float externalDistance = float.PositiveInfinity;
        /// <summary>
        /// The current distance
        /// </summary>
        public float distance {
            get {
                if (this.externalDistance < this.internalDistance)
                    return this.externalDistance;
                else
                    return this.internalDistance;
            }
        }
        protected bool distanceUpdated;

// #if UNITY_5_3_OR_NEWER
//         /// @copydoc Passer::RoboidControl::Thing::CreateComponent              
//         public override void CreateComponent() {
//             this.component = Unity.DistanceSensor.Create(this);
//             this.component.core = this;
//         }
// #endif        

        public override byte[] GenerateBinary() {
            if (!distanceUpdated)
                return Array.Empty<byte>();

            byte[] bytes = new byte[2];
            byte ix = 0;
            LowLevelMessages.SendFloat16(bytes, ref ix, this.internalDistance);
            distanceUpdated = false;
            return bytes;
        }

        /// <summary>
        /// Function to extract the distance received in the binary message
        /// </summary>
        /// <param name="bytes">The byte array</param>
        public override void ProcessBinary(byte[] bytes) {
            byte ix = 0;
            this.externalDistance = LowLevelMessages.ReceiveFloat16(bytes, ref ix);
        }

    }

}